import os
import sys
import struct

def fmdl(idn, ido, fn):
    fs = os.stat(fn).st_size
    b = open(fn, 'rb')
    
    print("[MTLTools] Working on file " + fn)

    #Grab actually important values from header
    b.seek(32)
    sec0 = struct.unpack("<I", b.read(4))[0] #Number of items in section 0
    sec1 = struct.unpack("<I", b.read(4))[0] #Number of items in section 1
    head = struct.unpack("<I", b.read(4))[0] #Header length
    b.seek(4, 1)
    sec1off = struct.unpack("<I", b.read(4))[0] #Sector 1 offset
    
    #Then start working on the rest of the file
    b.seek(64, 0)
    
    #Go through blockmap 0
    texoff = 0 #Texture offset
    texc = 0 #Texture count
    stroff = 0 #Str offset
    strc = 0 #Str count
    bcv = 0 #Block Check Value, make sure all blocks were found
    for i in range(sec0):
        dat = b.read(2)
        items = b.read(2)
        off = b.read(4)
        if(struct.unpack("<H", dat)[0] == 6): #Texture block
            texoff = struct.unpack("<I", off)[0]
            texc = struct.unpack("<H", items)[0]
            bcv = bcv + 1
        elif(struct.unpack("<H", dat)[0] == 12): #String block
            stroff = struct.unpack("<I", off)[0]
            strc = struct.unpack("<H", items)[0]
            bcv = bcv + 1
            
    #Go through blockmap 1
    strend = 0
    strlen = 0
    for i in range(sec1):
        dat = b.read(4)
        off = b.read(4)
        blen = b.read(4)
        if(struct.unpack("<I", dat)[0] == 3):
            strend = struct.unpack("<I", off)[0]
            strlen = struct.unpack("<I", blen)[0]
    
    #Check if all required blocks were found.
    ##NOTE: Change this when a new block is added
    if(bcv == 2):
        #We have all the required offsets now, start building output
        b.seek(head+stroff)
        
        #Store string lengths
        lens = []
        for i in range(strc):
            b.seek(2, 1)
            lens.append(struct.unpack('<H', b.read(2))[0])
            b.seek(4, 1)
        
        #Then process MTL
        strs = [] #Store strings in a list
        b.seek(sec1off + strend)
        for i in range(strc):
            ln = b.read(lens[i]+1).rstrip(b'\0').decode('utf-8')
            if(ln):
                strs.append(ln)
        
        #Scan textures
        b.seek(head + texoff)
        paths = [] #Store texture paths
        for i in range(texc):
            tex = struct.unpack("<H", b.read(2))[0] #Texture filename
            texp = struct.unpack("<H", b.read(2))[0] #Texture path
            if not(tex in paths):
                paths.append(tex)
                
        #Close file for now, we're done reading
        b.close()
        
        #Process the paths
        for path in paths:
            if(ido in strs[path-1]):
                strs[path-1] = strs[path-1].replace(ido, idn)     
                print("[MTLTools] Modified file " + fn)
                
        #Re-open file for writing
        b = open(fn, "r+b")
        
        #Seek to string block
        b.seek(sec1off + strend)
        #Re-write FMDL strings
        for s in strs:
            b.write(struct.pack("B", 0))
            b.write(bytes(s, 'utf-8'))
            
        #Done, close file for good and exit
        b.close()

def transfer(idn, ido, fn):
    idn = str(idn)
    ido = str(ido)
    
    for r, d, f in os.walk(fn): #Go through the whole folder
        for file in f:
            if('thumbs.db' in file.lower()): #Remove thumbs.db if found
                os.remove(os.path.join(r, file))
            elif (ido in file): #Edit any other file
                os.rename(os.path.join(r, file), os.path.join(r, file.replace(ido, idn)))
            elif('.fmdl' in file.lower()):
                fmdl(idn, ido, os.path.join(r, file))
        
    print("\nTeam aesthetics successfully transferred")
        
def main():
    fn = ''
    fnl = []
    a = 0
    for f in os.listdir('.'): #Look for directories in the folder
        if(os.path.isdir(f)):
            fnl.append(f)
            a = a+1
    if(a == 0): #No folders found
        input("Could not find an aesthetics folder, press any key to continue")
        sys.exit(1)
    elif(a == 1): #1 folder found, use it
        fn = fnl[0]
    elif(a > 1): #Multiple folders found, ask which one to use
        i = 1
        print("Found several folders, please choose which one to edit\n")
        for file in fnl:
            print(str(i) + ') ' + fnl[i-1])
            i = i+1
        num = input('\n')
        if(not(num.replace('-', '').isdigit()) or int(num) < 1 or int(num) > len(fnl)): #Parse and check the input number
            input("Invalid input")
            sys.exit(1)
        else:
            fn = fnl[int(num)-1]
            
    ido = int(input("\nCurrent Team ID: "))
    idn = int(input("\nNew Team ID: "))
    if(ido != 0 and idn != 0 and fn != ''):
        transfer(idn, ido, fn)
    
if __name__ == '__main__':
    main()